package cache

import (
	"context"

	"git.perx.ru/perxis/perxis-go/pkg/telemetry/metrics"
	"go.opentelemetry.io/otel/attribute"
	otelmetric "go.opentelemetry.io/otel/metric"
)

type telemetryMiddleware struct {
	next         Cache
	cacheMetrics *metrics.CacheMetrics
	attributes   otelmetric.MeasurementOption
}

// TelemetryMiddleware возвращает обертку над кэшем, которая используется для отслеживания количества хитов и промахов в кэше.
func TelemetryMiddleware(next Cache, name string, keyValues ...attribute.KeyValue) Cache {
	cacheMetrics, err := metrics.GetCacheMetrics()
	if err != nil {
		panic(err)
	}

	if name != "" {
		keyValues = append(keyValues, attribute.String("cache_name", name))
	}

	return &telemetryMiddleware{
		next:         next,
		cacheMetrics: cacheMetrics,
		attributes:   otelmetric.WithAttributes(keyValues...),
	}
}

func (c *telemetryMiddleware) Set(key, value any) error {
	return c.next.Set(key, value)
}

func (c *telemetryMiddleware) Get(key any) (any, error) {
	value, err := c.next.Get(key)
	if err != nil {
		c.cacheMetrics.MissesTotal.Add(context.TODO(), 1, c.attributes)
		return nil, err
	}
	c.cacheMetrics.HitsTotal.Add(context.TODO(), 1, c.attributes)
	return value, nil
}

func (c *telemetryMiddleware) Remove(key any) error {
	c.cacheMetrics.InvalidatesTotal.Add(context.TODO(), 1, c.attributes)
	return c.next.Remove(key)
}
