package collections

import (
	"testing"

	"git.perx.ru/perxis/perxis-go/pkg/optional"
	"git.perx.ru/perxis/perxis-go/pkg/schema"
	"git.perx.ru/perxis/perxis-go/pkg/schema/field"
	jsoniter "github.com/json-iterator/go"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestCollection_UnmarshalJSON(t *testing.T) {
	sch := schema.New("a", field.String()).WithMetadata(
		"collection_id", "collID",
		"collection_name", "collName")
	sch.ClearState()

	tests := []struct {
		name    string
		in      any
		expect  *Collection
		wantErr bool
	}{
		{name: "from schema",
			in:      sch,
			expect:  &Collection{Schema: sch, ID: "collID", Name: "collName"},
			wantErr: false},
		{name: "from collection",
			in:      &Collection{Schema: sch, ID: "id", Name: "name"},
			expect:  &Collection{Schema: sch, ID: "id", Name: "name"},
			wantErr: false},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			var c Collection
			b, _ := jsoniter.Marshal(tt.in)
			if err := jsoniter.Unmarshal(b, &c); (err != nil) != tt.wantErr {
				t.Errorf("UnmarshalJSON() error = %v, wantErr %v", err, tt.wantErr)
			}

			assert.True(t, c.Equal(tt.expect))
			assert.True(t, c.Schema.Equal(tt.expect.Schema))
		})
	}
}

func TestFromSchemaMetadata(t *testing.T) {
	testCases := []struct {
		name   string
		schema *schema.Schema
		want   *Collection
	}{
		{
			name:   "Nil",
			schema: nil,
			want:   nil,
		},
		{
			name:   "Empty",
			schema: &schema.Schema{},
			want:   &Collection{Schema: &schema.Schema{}},
		},
		{
			name:   "Without metadata",
			schema: schema.New("a", field.String()),
			want:   &Collection{Schema: schema.New("a", field.String())},
		},
		{
			name: "With metadata",
			schema: schema.New("a", field.String()).WithMetadata(
				"collection_id", "collID",
				"collection_name", "collName",
				"collection_single", "true",
				"collection_system", "true",
				"collection_nodata", "true",
				"collection_hidden", "true",
				"collection_no_publish", "true",
				"collection_no_archive", "true",
				"collection_view_space", "viewSpaceID",
				"collection_view_env", "viewEnvID",
				"collection_view_id", "viewCollID",
				"collection_view_filter", "viewFilter",
			),
			want: &Collection{
				ID:        "collID",
				Name:      "collName",
				Single:    optional.True,
				System:    optional.True,
				NoData:    optional.True,
				Hidden:    true,
				NoPublish: true,
				NoArchive: true,
				Schema:    schema.New("a", field.String()).WithMetadata("collection_id", "collID", "collection_name", "collName", "collection_single", "true", "collection_system", "true", "collection_nodata", "true", "collection_hidden", "true", "collection_no_publish", "true", "collection_no_archive", "true", "collection_view_space", "viewSpaceID", "collection_view_env", "viewEnvID", "collection_view_id", "viewCollID", "collection_view_filter", "viewFilter"),
				View: &View{
					SpaceID:      "viewSpaceID",
					EnvID:        "viewEnvID",
					CollectionID: "viewCollID",
					Filter:       "viewFilter",
				},
			},
		},
		{
			name: "With metadata revisions settings",
			schema: schema.New("a", field.String()).WithMetadata(
				"collection_id", "collID",
				"collection_name", "collName",
				"collection_no_revisions", "true",
				"collection_max_revisions", "10",
				"collection_revisions_ttl", "1h",
			),
			want: &Collection{
				ID:           "collID",
				Name:         "collName",
				NoRevisions:  true,
				MaxRevisions: 10,
				RevisionTTL:  3600000000000,
				Schema:       schema.New("a", field.String()).WithMetadata("collection_id", "collID", "collection_name", "collName", "collection_no_revisions", "true", "collection_max_revisions", "10", "collection_revisions_ttl", "1h"),
			},
		},
	}

	for _, tt := range testCases {
		t.Run(tt.name, func(t *testing.T) {
			result := FromSchema(tt.schema)
			require.Equal(t, tt.want, result)
		})
	}
}
