package zap

import (
	"context"
	"git.perx.ru/perxis/perxis-go/id"
	_ "git.perx.ru/perxis/perxis-go/id/system" // регистрируем обработчики для системных объектов
	"git.perx.ru/perxis/perxis-go/pkg/auth"
	"go.uber.org/zap"
	"go.uber.org/zap/zapcore"
)

type StringArray []string

func (ss StringArray) MarshalLogArray(arr zapcore.ArrayEncoder) error {
	for i := range ss {
		arr.AppendString(ss[i])
	}
	return nil
}

// Channels возвращает поле, содержащее список каналов, в которые должна быть передана запись.
func Channels(channels ...string) zap.Field {
	return zap.Field{
		Key:       channelKey,
		Type:      zapcore.SkipType, // используем тип zapcore.SkipType, чтобы при кодировании поле игнорировалось
		Interface: StringArray(channels),
	}
}

func Category(category string) zap.Field {
	return zap.String("category", category)
}

func Component(component string) zap.Field {
	return zap.String("component", component)
}

func Event(event string) zap.Field {
	return zap.String("event", event)
}

// Object возвращает поле и устанавливает передаваемый аргумент в качестве идентификатора объекта в формате ObjectId.
// Поддерживает типы в формате ObjectId: id.Descriptor, string, map[string]any, системные объекты.
func Object(v any) zap.Field {
	oid, _ := id.NewObjectId(v)
	return zap.Reflect("object", oid)
}

type callerConfig struct {
	object  any
	spaceID string
}

type CallerOption func(c *callerConfig)

func WithObject(object any) CallerOption {
	return func(c *callerConfig) {
		c.object = object
	}
}

func WithSpace(spaceID string) CallerOption {
	return func(c *callerConfig) {
		c.spaceID = spaceID
	}
}

// Caller возвращает поле и устанавливает в зависимости от переданных опций "вызывающего" в формате ObjectId.
func Caller(ctx context.Context, options ...CallerOption) zap.Field {
	c := new(callerConfig)
	for _, o := range options {
		o(c)
	}
	var oid *id.ObjectId
	if c.object != nil {
		oid, _ = id.NewObjectId(c.object)
	} else if ctx != nil {
		principal := auth.GetPrincipal(ctx)
		if accessor, ok := principal.(auth.SpaceAccessor); ok && c.spaceID != "" {
			principal = accessor.Space(c.spaceID)
		}
		oid, _ = id.NewObjectId(principal)
	}
	return zap.Reflect("caller", oid)
}

func Attr(attr any) zap.Field {
	return zap.Reflect("attr", attr)
}

func Tags(tags ...string) zap.Field {
	return zap.Array("tags", StringArray(tags))
}
